;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Yoshi's Island Dynamic Door Part 2 (locked door, blocked door)
; Programmed by SMWEdit/Edit1754
;
; Uses first extra bit: YES
; It will be a blocked door if it is set (a door with boards over it)
; It will be a locked door if it is NOT set (press up while holding a key)
;
; You will need to patch SMKDan's dsx.asm to your ROM with xkas
; this sprite, like all other dynamic sprites, uses the last 4 rows of sp4
;
; You will also need to patch my oampatch.asm to your ROM with xkas
; and use sprite header setting 10, because the locked door will spawn a regular door
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		DOOR_SPRITE = $79	; change this to the sprite number where you inserted the YI Door

		ACTSTATUS = $C2
		FXSPRITES = $1570

		EXTRA_BITS = $7FAB10

		EXTRABIT = $0E

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT and MAIN JSL targets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		dcb "INIT"
		RTL		

		dcb "MAIN"			
		PHB
		PHK				
		PLB				
		JSR SPRITE_ROUTINE	
		PLB
		RTL     


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPRITE_ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BITS		dcb %00000001,%00000010,%00000100,%00001000

ONYOSHI		LDA #$54		; \ play yoshi-spit sound
		STA $1DFC		; / effect (it fits the best)
RETURN1		RTS

NO_ENTER_1	JMP NO_ENTER

SPRITE_ROUTINE	JSR SUB_GFX		; call GFX routine
		LDA EXTRA_BITS,x	; \  skip unlock code
		AND #%00000100		;  | if the extra
		BNE RETURN1		; /  bit is set
		LDA $14C8,x		; \  return if sprite status != 8
		CMP #$08		;  |
		BNE RETURN1		; /
		LDA $9D			; \ return if sprites locked
		BNE RETURN1		; /
		JSR SUB_OFF_SCREEN_X0	; only process sprite while on screen

		LDA $16			; \  Don't begin opening
		AND #%00001000		;  | if it's not the first
		BEQ NO_ENTER_1		; /  frame of pressing up
		LDA $77			; \  don't go in
		AND #%00000100		;  | if mario is
		BEQ NO_ENTER_1		; /  not on ground
		LDY #$0C		; load number of times to go through loop
KEY_LOOP	CPY #$00		; \ zero? if so, that means that
		BEQ NO_ENTER_1		; / no sprites are carried keys
		DEY			; decrease # of times left+get index
		LDA $9E,y		; \  if the sprite
		CMP #$80		;  | is not a key,
		BNE KEY_LOOP		; /  go to next
		LDA $14C8,y		; \  if sprite is not
		CMP #$0B		;  | being carried, then
		BNE KEY_LOOP		; /  go to next sprite
		JSL $01A7DC		; \ if mario isn't touching
		BCC NO_ENTER_1		; / the door, don't go in
		LDA $187A		; \ if on Yoshi, you are not
		BNE ONYOSHI		; / allowed to pass through
		LDA #$01		; \ set going
		STA ACTSTATUS,x		; / in status
		LDA #$0F		; \ play opening
		STA $1DFC		; / sound effect
		LDY #$0C		; load number of times to go through loop
SPR_LOOP	CPY #$00		; \ zero? if so,
		BEQ END_SPR_LOOP	; / end loop
		DEY			; decrease # of times left+get index
		LDA $14C8,y		; \  if sprite is not
		CMP #$0B		;  | being carried,
		BNE SPR_LOOP		; /  then go to next sprite
		LDA #$00		; \ erase
		STA $14C8,y		; / shell
		PHX			; back up sprite index
		TYX			; this shell's index -> X
		PHY			; back up this shell's index
		LDA $D8,x		; \
		CMP $1C			;  | don't generate
		LDA $14D4,x		;  | if off screen
		SBC $1D			;  | vertically
		BNE ENDSMOKE		; /
		LDA $E4,x		; \
		CMP $1A			;  | don't generate
		LDA $14E0,x		;  | if off screen
		SBC $1B			;  | horizontally
		BNE ENDSMOKE		; /
		LDY #$03		; \
FINDFREE	LDA $17C0,y		;  | find a free
		BEQ FOUNDONE		;  | slot to
		DEY			;  | display effect
		BPL FINDFREE		;  | for smoke
		BRA ENDSMOKE		; /
FOUNDONE	LDA BITS,y		; \
		PHX			;  | mark this effect sprite
		LDX $15E9		;  | to be processed while the
		ORA FXSPRITES,x		;  | door is being opened and
		STA FXSPRITES,x		;  | sprites are locked
		PLX			; /
		LDA #$01		; \ set effect graphic to smoke graphic
		STA $17C0,y		; /
		LDA $E4,x 		; \ set x position for smoke
		STA $17C8,y		; /
		LDA $D8,x		; \ set y position for smoke
		STA $17C4,y		; /
		LDA #$1B		; \ set time to show smoke
		STA $17CC,y		; /
ENDSMOKE	PLY			; load backed up shell index
		PLX			; load backed up sprite index
		BRA SPR_LOOP		; go back to loop start
END_SPR_LOOP
		LDA #DOOR_SPRITE	; \ set custom
		STA $7FAB9E,x		; / sprite number
		JSL $0187A7		; get table values for custom sprite
		INC ACTSTATUS,x		; make door open
NO_ENTER

		JSL $01802A		; update position based on speed values
RETURN		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		DYNAMICSLOT = $03

TILES		dcb $00,$20,$00,$20
XDISP		dcb $F8,$F8,$08,$08
YDISP		dcb $F0,$00,$F0,$00
PROPS		dcb $00,$00,$40,$40

ENDSUB1		JMP ENDSUB

SUB_GFX		JSR GET_DRAW_INFO
		LDA #$00		; dynamic frame is always zero
		JSR GETSLOT		; reserve dynamic sprite slot
		BEQ ENDSUB1		; if no slots left, skip GFX
		STA DYNAMICSLOT		; set dynamic slot tile start
		PHX			; back up X
		LDX #$00		; load X with zero
TILELP		CPX #$04		; end of loop?
		BNE NORETFRML		; \ if so,
		JMP RETFRML		; / then end
NORETFRML	LDA XDISP,x		; get tile X offset
		CLC			; \ add it to sprite's
		ADC $00			; / screen X offset
		STA $0300,y		; set tile's X position
		LDA YDISP,x		; get tile Y offset
		CLC			; \ add it to sprite's
		ADC $01			; / screen Y offset
		STA $0301,y		; set tile's Y position
		PHX			; \
		LDX $15E9		;  | extra bit
		LDA EXTRA_BITS,x	;  | determines
		PLX			;  | which door
		AND #%00000100		;  | GFX to use
		LSR A			; /
		CLC			; \ add tile
		ADC TILES,x		; / offset
		CLC			; \ add in dynamic
		ADC DYNAMICSLOT		; / tile number
		STA $0302,y		; set tile #
		LDA PROPS,x		; get tile properties
		PHX			; back up X (index to tile data)
		LDX $15E9		; load X with index to sprite
		ORA $15F6,x		; load palette info
		PLX			; load backed up X
		ORA $64			; add in priority bits
		STA $0303,y		; set extra info
		INY			; \
		INY			;  | index to next slot
		INY			;  |
		INY			; /
		INX			; next tile to draw
		JMP TILELP		; loop
RETFRML		PLX			; load backed up X
		LDY #$02		; #$02 means the tiles are 16x16
		LDA #$03		; #$03 means 4 tiles were drawn
		JSL $01B7B3		; don't draw if offscreen, set sizes
ENDSUB		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Dynamic sprite routine
; Programmed mainly by SMKDan, but based on some of my code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TEMP = $09

SLOTPTR = $0660		;16bit pointer for source GFX
SLOTBANK = $0662	;bank
SLOTDEST = $0663	;VRAM address
SLOTSUSED = $06FE	;how many slots have been used

MAXSLOTS = $04		;maximum selected slots

SLOTS dcb $CC,$C8,$C4,$C0	;avaliable slots.  Any more transfers and it's overflowing by a dangerous amount.

GETSLOT
	PHY		;preserve OAM index
	PHA		;preserve frame
	LDA SLOTSUSED	;test if slotsused == maximum allowed
	CMP #MAXSLOTS
	BEQ NoneFree

	PLA		;pop frame
	REP #$20	;16bit A
	AND.w #$00FF	;wipe high
	XBA		;<< 8
	LSR A		;>> 1 = << 7
	STA TEMP	;back to scratch
	PEA SPRITEGFX	;push 16bit address, since i'm not sure how else i'm going to get the address of the label itself in TRASM
	PLA		;pull 16bit address
	CLC
	ADC TEMP	;add frame offset	
	STA SLOTPTR	;store to pointer to be used at transfer time
	SEP #$20	;8bit store
	PHB		;push db...
	PLA		;pull to A...
	STA SLOTBANK	;store bank to 24bit pointer

	LDY SLOTSUSED	;calculate VRAM address + tile number
	LDA SLOTS,y	;get tile# in VRAM
	PHA		;preserve for eventual pull
	SEC
	SBC #$C0	;starts at C0h, they start at C0 in tilemap
	REP #$20	;16bit math
	AND.w #$00FF	;wipe high byte
	ASL A		;multiply by 32, since 32 bytes/16 words equates to 1 32bytes tile
	ASL A
	ASL A
	ASL A
	ASL A
	CLC
	ADC #$0B44	;add 0B44, base address of buffer	
	STA SLOTDEST	;destination address in the buffer

	JSR DMABUFFER	;ROM -> RAM copy

	SEP #$20	;8bit A	
	INC SLOTSUSED	;one extra slot has been used

	PLA		;return starting tile number
	PLY
	RTS
		
NoneFree
	PLA
	PLY
	LDA #$00	;zero on no free slots
	RTS

;;;;;;;;;;;;;;;;
;Tansfer routine
;;;;;;;;;;;;;;;;

;DMA ROM -> RAM ROUTINE

DMABUFFER
;set destination RAM address
	REP #$20
	LDA SLOTDEST
	STA $2181	;16bit RAM dest
	LDY #$7F
	STY $2183	;set 7F as bank

;common DMA settings
	STZ $4300	;1 reg only
	LDY #$80	;to 2180, RAM write/read
	STY $4301

;first line
	LDA SLOTPTR
	STA $4302	;low 16bits
	LDY SLOTBANK
	STY $4304	;bank
	LDY #$80	;128 bytes
	STY $4305
	LDY #$01
	STY $420B	;transfer

;second line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR	;update source address
	CLC
	ADC #$0200	;512 bytes, next row
	STA SLOTPTR
	STA $4302	;low 16bits
	LDY SLOTBANK
	STY $4304	;bank
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B	;transfer

;third line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR	;update
	CLC
	ADC #$0200
	STA SLOTPTR
	STA $4302
	LDY SLOTBANK
	STY $4304
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B	;transfer

;fourth line
	LDA SLOTDEST	;update buffer dest
	CLC
	ADC #$0200	;512 byte rule for sprites
	STA SLOTDEST	;updated base
	STA $2181	;updated RAM address

	LDA SLOTPTR
	CLC
	ADC #$0200
	STA SLOTPTR
	STA $4302
	LDY SLOTBANK
	STY $4304
	LDY #$80
	STY $4305
	LDY #$01
	STY $420B

	SEP #$20	;8bit A
	RTS		;all done, return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS                     ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPRITEGFX
INCBIN sprites\yi_door_part2.bin